indexing

	author: "Dr. Thomas Tensi"
	date: "$Date$"
	revision: "$Revision$"

class
	BOARD

create
	make

feature {NONE} -- Initialization

	make (token_list : BOARD_TOKEN_LIST;  board_width : INTEGER) is
		-- Initialize `Current' to some random token configuration using
		-- tokens from `token_list' repeatedly until the board is filled;
		-- board width is set to `board_width'
		require
			board_width >= 7
		local
			i : INTEGER
			temp_array : ARRAY[BOARD_TOKEN]
		do
			width := board_width

			-- the TRIO board is square
			size := width * width
			create internal_array.make(1,  size)

			-- fill some temporary array with a regular token distribution
			create temp_array.make(1,  size)
			token_list.start

			from
				i := 1
			until
				i > size
			loop
				temp_array[i] := token_list.item
				i := i + 1
				token_list.forth
			end

			create reference_list.make
			reference_list.compare_objects
			reference_list.fill(temp_array)

			-- fill TRIO board with some random configuration
			reset
		end


feature -- Access

    width : INTEGER    -- width of the board in one dimension

	-- --------------------

    item (position : BOARD_POSITION) : BOARD_TOKEN is
    		-- tells what token is located at `position'
    	do
			Result := internal_array[get_index(position)]
    	end


feature -- Status report

	position_is_solution (position : BOARD_POSITION;  value : INTEGER) : BOOLEAN is
			-- tells whether `position' is center of some TRIO solution for `value'
		local
			solution_list : LIST[STRING]
		do
			solution_list := get_solution_expressions(position, value)
			Result := (solution_list.count > 0)
		end

	-- --------------------

	get_solution_expressions (position : BOARD_POSITION; value : INTEGER) : LIST[STRING] is
			-- gives all solutions for `position' as center of some TRIO solution
			-- for `value' as a list of equation strings
		local
			first_position : BOARD_POSITION
			first_value : INTEGER
			i : INTEGER
			is_okay : BOOLEAN
			j : INTEGER
			last_position : BOARD_POSITION
			last_value : INTEGER
			middle_value : INTEGER
			operation_char : CHARACTER
			st : STRING
			string_list : LINKED_LIST[STRING]
		do
			create string_list.make
			middle_value := item(position).value

			from
				i := -1
			until
				i > 1
			loop
				from
					j := -1
				until
					j > 1
				loop
					is_okay := ((i /= 0 or j /=0)
					            and then is_acceptable_coordinate(position.row + i)
						        and then is_acceptable_coordinate(position.row - i)
						        and then is_acceptable_coordinate(position.column + j)
						        and then is_acceptable_coordinate(position.column - j))

					if is_okay then
						create first_position.make(position.row + i, position.column + j)
						first_value := item(first_position).value
						create last_position.make(position.row - i, position.column - j)
						last_value := item(last_position).value

						if first_value * middle_value + last_value = value then
							operation_char := '+'
						elseif first_value * middle_value - last_value = value then
							operation_char := '-'
						else
							-- this is no TRIO solution
							operation_char := ' '
						end

						if operation_char /= ' ' then
							create st.make_empty
							st.append_integer(first_value)
							st.append("  ")
							st.append_integer(middle_value)
							st.append_character(' ')
							st.append_character(operation_char)
							st.append_character(' ')
							st.append_integer(last_value)
							string_list.extend(st)
						end
					end

					j := j + 1
				end

				i := i + 1
			end

			Result := string_list
		end

	-- --------------------

	get_solving_positions (value : INTEGER) : LIST[BOARD_POSITION] is
			-- returns list of solving positions for `value'
		local
			i : INTEGER
			j : INTEGER
			position : BOARD_POSITION
			position_list : LINKED_LIST[BOARD_POSITION]
			solution_expression_list : LIST[STRING]
		do
			create position_list.make

			from
				i := 1
			until
				i = width + 1
			loop
				from
					j := 1
				until
					j = width + 1
				loop
					create position.make(i, j)
					solution_expression_list := get_solution_expressions(position, value)

					if solution_expression_list.count > 0 then
						position_list.extend(position)
					end

					j := j + 1
				end

				i := i + 1
			end

			Result := position_list
		end


feature -- Element change

	reset is
			-- fills board with some random token configuration;
			-- `is_simple_game' tells whether zeroes occur in
			-- configuration
		local
			current_time : TIME
			i : INTEGER
			j : INTEGER
			rnd : RANDOM
			temp_array : ARRAYED_LIST[BOARD_TOKEN]
		do
			-- select randomly from temporary array and shorten its length accordingly
			create current_time.make_now
			create rnd.set_seed(current_time.seconds)
			rnd.start

			create temp_array.make(size)
			temp_array.start
			temp_array.fill(reference_list)

			from
				i := size
			until
				i = 0
			loop
				rnd.forth
				j := (rnd.real_item * i).floor + 1
				internal_array[i] := temp_array[j]
				temp_array[j] := temp_array[i]
				i := i - 1
			end
		ensure
			is_consistent : is_consistent_board
		end


feature -- Conversion

	to_string : STRING is
			-- converts `Current' to STRING
		local
			i : INTEGER
			j : INTEGER
			position : BOARD_POSITION
			st : STRING
		do
			create st.make_empty

			from
				i := 1
			until
				i > width
			loop
				from
					j := 1
				until
					j > width
				loop
					create position.make(i, j)

					st.append(item(position).to_string)

					if j < width then
						st.append(" ")
					end

					j := j + 1
				end

				if i < width then
					st.append(" / ")
				end

				i := i + 1
			end

			Result := st
		end


feature {NONE} -- Implementation

    size : INTEGER -- number of tokens on TRIO board
	internal_array : ARRAY[BOARD_TOKEN]   -- linear array of TRIO tokens

	reference_list : SORTED_TWO_WAY_LIST[BOARD_TOKEN]
	  -- sorted linear list of all tokens on board

	-- --------------------

	get_index (position : BOARD_POSITION) : INTEGER is
			-- get index into `internal_array' for `position'
		do
			-- information is stored in row-major order in linear array
			Result := (position.row - 1) * width + position.column
		end


	-- --------------------

	is_acceptable_coordinate (coordinate : INTEGER) : BOOLEAN is
			-- tells whether `coordinate' is on board
		do
			Result := (1 <= coordinate and coordinate <= width)
		end

	-- --------------------

	is_consistent_board : BOOLEAN is
			-- checks whether `Current' is filled consistently with TRIO tokens
		local
			expected_value : BOARD_TOKEN
			i : INTEGER
			is_okay : BOOLEAN
			sorted_list : SORTED_TWO_WAY_LIST[BOARD_TOKEN]
		do
			create sorted_list.make
			sorted_list.compare_objects
			sorted_list.fill(internal_array)

			sorted_list.start
			reference_list.start
			is_okay := (sorted_list.count = size)

			from
				i := 1
			until
				i > size or not is_okay
			loop
				expected_value := reference_list.item
				is_okay := (sorted_list.item = expected_value)
				reference_list.forth
				sorted_list.forth
				i := i + 1
			end

			Result := is_okay
		end


invariant
	invariant_clause: True -- Your invariant here

end
