indexing
	description: "User Preferences for TRIO Game"
	author: "Dr. Thomas Tensi"
	date: "$Date$"
	revision: "$Revision$"

class
	GAME_PREFERENCES

inherit
	WEL_REGISTRY
		export {NONE}
			all
		end


create
	make

feature {NONE} -- Initialization

	make is
			-- Initialize a new preferences object.
		do
			board_size := get_int_value(board_size_variable_name, default_board_size)
			language := get_string_value(language_variable_name, default_language)
			sound_is_on := get_bool_value(sound_variable_name, default_sound_is_on)
			think_time := get_int_value(think_time_variable_name, default_think_time)
			unsolvable_button_is_shown :=
			  get_bool_value(unsolvable_variable_name, default_unsolvable_button_is_shown)
			zeroes_are_allowed :=
			  get_bool_value(zeroes_variable_name, default_zeroes_are_allowed)
		end


feature -- Access

	board_size : INTEGER
		-- number of rows on square board

    language : STRING
    	-- user interface language

	sound_is_on : BOOLEAN
		-- tells whether sounds are used

	think_time : INTEGER
		-- think time in seconds

	unsolvable_button_is_shown : BOOLEAN
		-- tells whether button for 'no solutions' is shown

	zeroes_are_allowed : BOOLEAN
		-- tells whether zero tokens may occur on board


feature -- Basic operations

	put_settings (new_board_size : INTEGER;  new_think_time : INTEGER;
	              new_zeroes_are_allowed : BOOLEAN;  new_sound_is_on : BOOLEAN;
	              new_unsolvable_button_is_shown : BOOLEAN;  new_language : STRING)
			-- Adapt preferences settings.
		do
			board_size                 := new_board_size
			language                   := new_language
			sound_is_on                := new_sound_is_on
			think_time                 := new_think_time
			unsolvable_button_is_shown := new_unsolvable_button_is_shown
			zeroes_are_allowed         := new_zeroes_are_allowed

			-- put it into Windows registry
			set_string_value(board_size_variable_name, board_size.out)
			set_string_value(language_variable_name, language.out)
			set_string_value(sound_variable_name, sound_is_on.out)
			set_string_value(think_time_variable_name, think_time.out)
			set_string_value(unsolvable_variable_name,unsolvable_button_is_shown.out)
			set_string_value(zeroes_variable_name, zeroes_are_allowed.out)
		end


feature {NONE} -- Implementation

	-- --------------------
    -- default values
	-- --------------------

	default_board_size                 : INTEGER is 9
	default_language                   : STRING is
		once result := "English" end
	default_sound_is_on                : BOOLEAN is true
	default_think_time                 : INTEGER is 25
	default_unsolvable_button_is_shown : BOOLEAN is false
	default_zeroes_are_allowed         : BOOLEAN is false

	-- ---------------------------
	-- registry path and variables
	-- ---------------------------

	registry_path : STRING is
		once result := "HKEY_CURRENT_USER\Software\Tensi Software\Trio" end
		-- path in registry to trio preferences

	board_size_variable_name : STRING is
		once result := "board_size" end
		-- variable in registry taking board size

	language_variable_name : STRING is
		once result := "language" end
		-- variable in registry taking preferred language

	sound_variable_name : STRING is
		once result := "sound_is_on" end
		-- variable in registry taking flags for sound

	think_time_variable_name : STRING is
		once result := "think_time" end
		-- variable in registry taking think time

	unsolvable_variable_name : STRING is
		once result := "unsolvable_button_is_shown" end
		-- variable in registry telling whether the unsolvable button is shown

	zeroes_variable_name : STRING is
		once result := "zeroes_are_allowed" end
		-- variable in registry taking flags for zero tokens

	-- --------------------

	get_bool_value (variable_name : STRING; default_value : BOOLEAN) : BOOLEAN is
			-- get data from preferences registry variable with `variable_name'
			-- and use `default_value' if not available
		local
			st : STRING
		do
			st := get_string_value(variable_name, default_value.out)
			Result := st.to_string_8.to_boolean
		end

	-- --------------------

	get_int_value (variable_name : STRING; default_value : INTEGER) : INTEGER is
			-- get data from preferences registry variable with `variable_name'
			-- and use `default_value' if not available
		local
			st : STRING
		do
			st := get_string_value(variable_name, default_value.out)
			Result := st.to_string_8.to_integer
		end

	-- --------------------

	get_string_value (variable_name : STRING; default_value : STRING)
		 : STRING is
			-- get data from preferences registry variable with `variable_name'
			-- and use `default_value' if not available
		local
			registry_value : WEL_REGISTRY_KEY_VALUE
		do
			registry_value := open_key_value(registry_path, variable_name)

			if registry_value = void then
				Result := default_value
			else
				Result := registry_value.string_value
			end
		end

	-- --------------------

	set_string_value (variable_name : STRING; value : STRING) is
			-- sets preferences registry variable with `variable_name' to
			-- `value'
		local
			registry_type  : WEL_REGISTRY_KEY_VALUE_TYPE
			registry_value : WEL_REGISTRY_KEY_VALUE
		do
			create registry_type
			create registry_value.make(registry_type.reg_sz, value)
			save_key_value(registry_path, variable_name, registry_value)
		end


end
