indexing
	description: "Presentation of a TRIO Board"
	author: "Dr. Thomas Tensi"
	date: "$Date$"
	revision: "$Revision$"

class
	BOARD_WINDOW

inherit
	WEL_CONTROL_WINDOW
		rename
			make as original_make
		redefine
			class_background,
			on_left_button_down,
			on_mouse_move,
			on_paint,
			on_right_button_down
		end

	WEL_STANDARD_COLORS
		export
			{NONE} all
		end


create
	make


feature {NONE} -- Initialization

	make (parent_window : WEL_WINDOW) is
			-- initializes `Current' for `board'
		do
			original_make(parent_window, "")
			create tool_tip.make_child(current, "")
			-- tool_tip.
		end


feature -- Element change

	set_board (current_board : BOARD) is
			-- sets associated board to `current_board'
		do
			board := current_board
			create {LINKED_LIST[BOARD_POSITION]} marked_position_list.make
		end

	-- --------------------

	set_special_positions (position_list : LIST[BOARD_POSITION]; value : INTEGER;
	                       is_good : BOOLEAN) is
			-- sets positions on board given as `position_list' as good
			-- or bad (depending on `is_good') relative to required `value'
		require
			not_void : position_list /= void
		do
			marked_position_list := position_list
			marked_position_list.compare_objects
			is_marked_as_good    := is_good
			required_value       := value
			invalidate
		end


feature {NONE} -- Event handling

	notify_parent_about_click (x_pos, y_pos : INTEGER;  is_right_click : BOOLEAN) is
			-- tell parent main window that some click has occured at (`x_pos',`y_pos')
		local
			position: BOARD_POSITION
			main_window : MAIN_WINDOW
		 do
			position := position_at_coordinate(x_pos, y_pos)

			if position /= void then
				main_window ?= parent

				if main_window /= void then
					main_window.notify_about_click(is_right_click, position)
				end
			end
		 end

	-- --------------------

	on_left_button_down (keys, x_pos, y_pos: INTEGER) is
		do
			notify_parent_about_click(x_pos, y_pos, false)
		end

	-- --------------------

	on_mouse_move (keys, x_pos, y_pos: INTEGER) is
			-- show solution expressions when on valid solution
		local
			message_is_updated : BOOLEAN
			old_message : STRING
			old_message_is_nonempty : BOOLEAN
			parent_window : MAIN_WINDOW
			position : BOARD_POSITION
			solution_expression_list : LIST[STRING]
			st : STRING
		do
			parent_window ?= parent
		   	old_message := parent_window.displayed_message

		   	old_message_is_nonempty :=
		   		old_message /= void and then old_message.count > 0

			if marked_position_list = void or else marked_position_list.count = 0
			   or else not is_marked_as_good then
			   	st := ""
			   	message_is_updated := old_message_is_nonempty
			else
			   	position := position_at_coordinate(x_pos, y_pos)

			   	if not marked_position_list.has(position) then
				   	st := ""
				   	message_is_updated := old_message_is_nonempty
			   	else
			   		-- construct tooltip from
			   		solution_expression_list :=
			   			board.get_solution_expressions(position, required_value)

			   		from
			   			solution_expression_list.start
			   			create {STRING_32}st.make_empty
			   		until
			   			solution_expression_list.after
			   		loop
			   			if not st.is_empty then
			   				st.append(" / ")
			   			end

			   			st.append(solution_expression_list.item)
			   			solution_expression_list.forth
			   		end

				   	message_is_updated := true
			   	end
			end

			if message_is_updated then
				parent_window.show_message(st)
			end
		end

	-- --------------------

	on_paint (paint_dc: WEL_PAINT_DC; invalid_rect: WEL_RECT) is
			-- paints the board
		local
			board_width : INTEGER
			column : INTEGER
			ellipse_rx : INTEGER
			ellipse_ry : INTEGER
			fill_color : WEL_COLOR_REF
			position : BOARD_POSITION
			row : INTEGER
			token_string : STRING
		do
			if board /= void then
				board_width := board.width

				cell_width  := width  / (1 + board_width)
				cell_height := height / (1 + board_width)

				ellipse_rx := (cell_width  / (1 + space_percentage)).floor // 2
				ellipse_ry := (cell_height / (1 + space_percentage)).floor // 2

				-- initialize the token color, frame pen and font
				update_device_context(paint_dc, (16 * ellipse_rx) // 10)

				-- draw all tokens
				from
					row := 1
				until
					row = board_width + 1
				loop
					from
						column := 1
					until
						column = board_width + 1
					loop
						create position.make(row, column)
						token_string := board.item(position).to_string

						-- find color of token
						if not marked_position_list.has(position) then
							fill_color := yellow
						elseif is_marked_as_good then
							fill_color := green
						else
							fill_color := red
						end

						draw_token(paint_dc, token_string, center_x(column), center_y(row),
						           ellipse_rx, ellipse_ry, fill_color);
						column := column + 1
					end

					row := row + 1
				end
			end
		end

	-- --------------------

	on_right_button_down (keys, x_pos, y_pos: INTEGER) is
		do
			notify_parent_about_click(x_pos, y_pos, true)
		end


feature {NONE} -- Transformation

	center_x (column : INTEGER) : INTEGER is
			-- the x-coordinate in window for `column' where the center of a token
			-- is located
		do
			Result := (cell_width * (space_percentage + 0.5)
			           + (column - 1) * cell_width).floor
		end

	-- --------------------

	center_y (row : INTEGER) : INTEGER is
			-- the y-coordinate in window for `row' where the center of a token
			-- is located
		do
			Result := (cell_height * (space_percentage + 0.5)
			          + (row - 1) * cell_height).floor
		end

	-- --------------------

	position_at_coordinate (x_coord : INTEGER;  y_coord : INTEGER) : BOARD_POSITION is
			-- the position on board at coordinate (`x_coord',`y_coord') or Void
		local
			column   : INTEGER
			position : BOARD_POSITION
			row      : INTEGER
		do
			column := ((x_coord - cell_width * space_percentage) / cell_width).floor + 1
			row    := ((y_coord - cell_height * space_percentage) / cell_height).floor + 1
			create position.make(1,1)

			if position.is_in_range(column) and position.is_in_range(row) then
				create position.make(row, column)
			else
				position := Void
			end

			Result := position
		end


feature {NONE} -- Implementation

	board : BOARD  -- associated board

	marked_position_list : LIST[BOARD_POSITION]   -- list of specially marked positions
	required_value : INTEGER                      -- required value
	is_marked_as_good : BOOLEAN

	-- tool tip
	tool_tip : WEL_POPUP_WINDOW

	-- presentation parameters
	cell_height      : DOUBLE	-- height of single token cell in pixels
	cell_width       : DOUBLE   -- width of single token cell in pixels
    space_percentage : DOUBLE is 0.3 -- the relative width of white space around tokens

	-- --------------------

	class_background: WEL_BRUSH is
			-- White background
		once
			create Result.make_solid(white)
		end

	-- --------------------

	draw_token (dc : WEL_DC;  st : STRING;
	            cx, cy : INTEGER;  rx, ry : INTEGER;  fill_color : WEL_COLOR_REF) is
			-- draws an ellipse with center at (`cx',`cy') and
			-- radius (`rx',`ry') with text `st' and `fill_color' into windows
			-- device context `dc'
		local
			brush : WEL_BRUSH
			rectangle : WEL_RECT
		do
			create brush.make_solid(fill_color)
			dc.select_brush(brush)
			create rectangle.make(cx - rx, cy - ry, cx + rx, cy + ry)
			dc.ellipse(rectangle.x, rectangle.y, rectangle.right, rectangle.bottom)
			dc.draw_centered_text(st, rectangle)
		end

	-- --------------------

	update_device_context (dc : WEL_DC; font_height : INTEGER) is
			-- sets device context `dc' to appropriate value for pen and font
		local
			logical_font : WEL_LOG_FONT
			new_font : WEL_FONT
			pen: WEL_PEN
		do
			create pen.make_solid(2, black)
			dc.select_pen(pen)
			create logical_font.make(font_height, "Arial")
			create new_font.make_indirect(logical_font)
			dc.select_font(new_font)
			dc.set_background_transparent
		end


invariant
	invariant_clause: True -- Your invariant here

end
